use crate::{generate_completions, Opt, RunMode, TestingPic};
use music_bg_lib::background::set_wallpaper;
use music_bg_lib::config::Config;
use music_bg_lib::dbus_interface::player_dbus::setup_connection;
use music_bg_lib::img_processors::process_image;
use music_bg_lib::result::MBGResult;
use std::io::Read;
use std::time::Duration;

pub fn start_listen() -> MBGResult<()> {
    let config = Config::get_props();
    debug!("Found config: \n{:#?}", config);
    let connection = setup_connection()?;

    loop {
        connection.process(Duration::from_millis(100)).unwrap();
    }
}

pub fn test_processors(test_data: TestingPic) -> MBGResult<()> {
    let mut img_file = std::fs::File::open(test_data.input.as_str())?;
    let mut image_bytes = Vec::new();
    img_file.read_to_end(&mut image_bytes)?;
    let res = process_image(image_bytes)?;
    res.save(test_data.output.as_str())?;
    if test_data.wallpaper {
        set_wallpaper(test_data.output.as_str());
    }
    Ok(())
}

pub fn run(opts: Opt) -> MBGResult<()> {
    let mode = opts.cmd.unwrap_or_else(|| RunMode::Run);
    match mode {
        RunMode::Run => start_listen(),
        RunMode::GenConf => Config::generate_config(),
        RunMode::TestPic(data) => test_processors(data),
        RunMode::GenComp(shell) => generate_completions(shell),
    }
}
