use std::process::Command;

use dbus::blocking::Connection;
use dbus::Message;

use crate::config::Config;
use crate::img_processors::process_image;
use crate::result::MBGResult;

pub fn reset_background_handler(_: (), _: &Connection, message: &Message) -> bool {
    let member_name = message.read1::<String>();
    if let Ok(name) = member_name {
        if name.starts_with("org.mpris.MediaPlayer2.") {
            reset_bg();
        }
    }
    true
}

pub fn reset_bg() {
    info!("Reset BG");
    let config = Config::get_props();
    let out = Command::new("sh")
        .arg("-c")
        .arg(config.reset_command.as_str())
        .output();
    if let Err(restore_error) = out {
        error!("Can't restore background: {}", restore_error.to_string());
    }
}

pub fn process_image_url(image_url: String) -> MBGResult<()> {
    let resp = attohttpc::get(image_url.as_str()).send()?;
    let bytes_vec = resp.bytes()?;
    let res = process_image(bytes_vec)?;
    let img_path = "/tmp/music_bg.png";
    res.save(img_path)?;
    set_wallpaper(img_path);
    Ok(())
}

pub fn set_wallpaper(path: &str) {
    debug!("Setting background");
    let config = Config::get_props();
    let out = Command::new("sh")
        .arg("-c")
        .arg(config.set_command.replace(r"{}", path))
        .output();
    if let Err(set_error) = out {
        error!("Can't set background: {}", set_error.to_string());
    }
}
