pub type MBGResult<T> = Result<T, MBGError>;

#[derive(Debug, Fail)]
pub enum MBGError {
    #[fail(display = "{}", _0)]
    StdErr(String),
    #[fail(display = "D-BUS error: {}", _0)]
    DBusErr(String),
    #[fail(display = "Configuration file parsing error: {}", _0)]
    ConfigErr(String),
    #[fail(display = "Invalid screen resolution string: {}", _0)]
    ResolutionFormatErr(String),
    #[fail(display = "Error while processing metadata from D-BUS: {}", _0)]
    MetadataError(String),
    #[fail(display = "Error while fetching image: {}", _0)]
    RequestError(String),
    #[fail(display = "Image processing error: {}", _0)]
    ImageError(String),
}

impl From<std::io::Error> for MBGError {
    fn from(err: std::io::Error) -> Self {
        Self::StdErr(err.to_string())
    }
}

impl From<std::num::ParseIntError> for MBGError {
    fn from(err: std::num::ParseIntError) -> Self {
        MBGError::StdErr(err.to_string())
    }
}

impl From<std::str::Utf8Error> for MBGError {
    fn from(err: std::str::Utf8Error) -> Self {
        MBGError::StdErr(err.to_string())
    }
}

impl From<toml::de::Error> for MBGError {
    fn from(err: toml::de::Error) -> Self {
        Self::ConfigErr(err.to_string())
    }
}

impl From<toml::ser::Error> for MBGError {
    fn from(err: toml::ser::Error) -> Self {
        Self::ConfigErr(err.to_string())
    }
}

impl From<dbus::Error> for MBGError {
    fn from(err: dbus::Error) -> Self {
        Self::DBusErr(err.to_string())
    }
}

impl From<attohttpc::Error> for MBGError {
    fn from(err: attohttpc::Error) -> Self {
        Self::RequestError(err.to_string())
    }
}

impl From<image::error::ImageError> for MBGError {
    fn from(err: image::error::ImageError) -> Self {
        MBGError::ImageError(err.to_string())
    }
}
